package com.lofiminds.xwung
{
   import com.lofiminds.gm.CollisionContext;
   import com.lofiminds.gm.GameContext;
   import com.lofiminds.gm.GameGraphics;
   import com.lofiminds.gm.GameImage;
   import com.lofiminds.gm.GameInput;
   import com.lofiminds.gm.GameObject;
   import com.lofiminds.gm.GameUtil;
   import com.lofiminds.gm.Instances;
   import flash.display.Bitmap;
   import flash.geom.ColorTransform;
   import flash.media.Sound;
   import flash.ui.Mouse;
   
   public class Player extends GameObject
   {
      
      private static var GxClass:Class = Player_GxClass;
      
      private static var gameImage:GameImage = new GameImage(Bitmap(new GxClass()).bitmapData);
      
      private static var GxClass2:Class = Player_GxClass2;
      
      private static var snd1Class:Class = Player_snd1Class;
      
      private static var enemyTypes:Array = new Array(Enemy1,Enemy2,Enemy3);
      
      private static var chainImages:Array = initChainImages();
      
      private static var chainImage:GameImage = new GameImage(Bitmap(new GxClass2()).bitmapData);
      
      internal static var explodePlayerSnd:Sound = Sound(new snd1Class());
       
      
      private var hitbox:GameObject;
      
      private var check:uint;
      
      private var gp:Gameplay;
      
      private var bonustime:int = 0;
      
      private var invincible:int;
      
      public function Player()
      {
         super();
         setImage(gameImage);
         useCustomDraw = true;
         depth = 2;
         invincible = 100;
         gp = Instances.getFirst(Gameplay) as Gameplay;
         gp.level = 0.02;
         hitbox = Instances.create(Hitbox,x,y);
      }
      
      private static function initChainImages() : Array
      {
         var _loc4_:uint = 0;
         var _loc1_:Array = new Array(10);
         var _loc2_:ColorTransform = new ColorTransform();
         var _loc3_:int = 0;
         while(_loc3_ < _loc1_.length)
         {
            _loc4_ = GameUtil.make_color_hsv(GameUtil.random() * 42,1 - GameUtil.random() * 0.39,1);
            GameUtil.setColorTransform(_loc4_,_loc2_);
            _loc1_[_loc3_] = chainImage.clone().colorTransform(_loc2_);
            _loc3_++;
         }
         return _loc1_;
      }
      
      override public function update() : void
      {
         var _loc1_:Number = NaN;
         var _loc2_:Class = null;
         super.update();
         if(int(this.x) != GameInput.mouseX)
         {
            x = GameInput.mouseX - 0.5 + Math.random();
         }
         if(int(this.y) != GameInput.mouseY)
         {
            y = GameInput.mouseY - 0.5 + Math.random();
         }
         hitbox.x = x;
         hitbox.y = y;
         if(GameUtil.frac(gp.level) > 0.0075)
         {
            GameUtil.setRandomSeed(check);
            check += 17;
            if(Math.round(GameUtil.random() * (35 - gp.level * 0.1)) == 1)
            {
               gp.level += 0.02;
               _loc1_ = GameUtil.random() * (Math.PI * 2);
               _loc2_ = Enemy1;
               if(gp.level > 5)
               {
                  _loc2_ = enemyTypes[int(Math.random() * enemyTypes.length)];
               }
               else if(gp.level > 4)
               {
                  _loc2_ = enemyTypes[int(Math.random() * (enemyTypes.length - 1))];
               }
               Instances.create(_loc2_,320 + GameUtil.lengthdir_x(420,_loc1_),240 + GameUtil.lengthdir_y(420,_loc1_));
            }
            GameUtil.randomize();
         }
         if(GameUtil.frac(gp.level) < 0.0075)
         {
            if(!Instances.exists(Boss))
            {
               if(!Instances.exists(Enemy1) && !Instances.exists(Enemy2) && !Instances.exists(Enemy3))
               {
                  Instances.create(Boss,320,-180);
               }
            }
         }
         if(invincible > 0)
         {
            --invincible;
         }
         if(bonustime > 0)
         {
            if(--bonustime == 0)
            {
               gp.hasPowerup = false;
            }
         }
      }
      
      override public function getClass() : Class
      {
         return Player;
      }
      
      override public function onCollision(param1:CollisionContext) : void
      {
         if(param1.other is Bonus)
         {
            gp.hasPowerup = true;
            this.bonustime = 1000;
            Instances.destroy(param1.other);
         }
         else if(invincible == 0)
         {
            Instances.destroy(this);
         }
      }
      
      override public function onCustomDraw(param1:GameGraphics) : Boolean
      {
         if((invincible & 8) != 0)
         {
            return false;
         }
         var _loc2_:GameObject = Instances.getFirst(Sledge);
         if(_loc2_ != null)
         {
            drawChain(param1,_loc2_.x,_loc2_.y);
         }
         return true;
      }
      
      override public function destroy() : void
      {
         GameContext.app.playSound(explodePlayerSnd);
         Instances.create(Rings,x,y);
         Instances.destroyAll(Sledge);
         Instances.destroyAll(Hitbox);
         gp.onGameOver();
         Mouse.show();
         super.destroy();
      }
      
      private function drawChain(param1:GameGraphics, param2:Number, param3:Number) : void
      {
         var _loc4_:Number = GameUtil.point_direction(param2,param3,this.x,this.y);
         var _loc5_:Number = GameUtil.lengthdir_x(14,_loc4_);
         var _loc6_:Number = GameUtil.lengthdir_y(14,_loc4_);
         var _loc7_:int = int(GameUtil.point_distance(param2,param3,this.x,this.y) * (1 / 14));
         var _loc8_:Number = 1;
         var _loc9_:int = 1;
         while(_loc9_ < _loc7_)
         {
            param2 += _loc5_;
            param3 += _loc6_;
            if(gp.hasPowerup)
            {
               _loc8_ = !!(_loc9_ & 1 == 1) ? 1 : 2;
            }
            param1.drawGameImage(GameImage(chainImages[int(Math.random() * chainImages.length)]),0,param2,param3,0,null,_loc8_);
            _loc9_++;
         }
      }
   }
}
